' BGInfo and TSBackdrop installation script.

' For Terminal/Citrix Servers we are using a combination of SystemInternal's BgInfo and
' CTRL-ALT-DEL's TSbackdrop to create a nice informative wallpaper for all Terminal/Citrix
' server users. The BGInfo.bmp file is placed in their %TEMP% folder and the TSBKDRP.BMP
' is placed in their "My Documents" or "Personal" folder.

' Before we proceed we first need to check that the "Terminal Services" (TermServ) service
' is running, otherwise using the WMI provider to query the Win32_TerminalServiceSetting
' properties will return a null value.

' I found the need to add these functions for Terminal/Citrix servers that are in a Novell
' only (workgroup) environment, and not part of a Domain, as this changes the way in which
' some of the services will start. So without it, if using the WMI provider to query the
' properties of the Win32_TerminalServiceSetting class during an automated build process,
' the returned value would not be consistent for non-Domain members when executing tasks
' from the RunOnce, RunOnceEX, and GUIRunOnce locations. I have found that it's in a "Start
' Pending" state. This script will wait until it's in a "Running" state before proceeding.

' The script has also been structured to deploy to BGInfo only to non Terminal/Citrix
' servers. In this case, the BGInfo.bmp file is placed in the %SystemRoot% folder.

' Version 1.3 released on 21st October 2009.
' Written by Jeremy@jhouseconsulting.com on 26th November 2008.

Option Explicit

Dim objFSO, objFolder, wshShell, oShellLink, strCitrixBGI, strSystemRoot
Dim strProgramFiles, strAUStartup, strScriptPath, strProgramFolder
Dim strCommandLine, strService, blnRunning, blnInstalled, blnReturn

set WshShell = WScript.CreateObject("WScript.Shell")
set objFSO = CreateObject("Scripting.FileSystemObject")

strCitrixBGI = "Citrix"

strService = "TermService"

strSystemRoot = WshShell.ExpandEnvironmentStrings("%SystemRoot%")
strProgramFiles = WshShell.ExpandEnvironmentStrings("%ProgramFiles%")
strAUStartup = WshShell.SpecialFolders("AllUsersStartup")
strScriptPath = Left(WScript.ScriptFullName, InstrRev(WScript.ScriptFullName, "\"))

'Create the Program Files folder
strProgramFolder = strProgramFiles & "\BGinfo"
If objFSO.FolderExists(strProgramFolder) Then
  Set objFolder = objFSO.GetFolder(strProgramFolder)
'  WScript.Echo strProgramFolder & " already exists."
Else
  Set objFolder = objFSO.CreateFolder(strProgramFolder)
'  WScript.Echo "Created " & strProgramFolder
End If

Select Case lcase(CheckServiceStatus(strService))
  Case "running"
    blnInstalled = True
    blnRunning = True
  Case "start pending"
    blnInstalled = True
    blnRunning = False
  Case "not installed"
    blnInstalled = False
    blnRunning = False
  Case Else
    blnInstalled = False
    blnRunning = False
End Select

If blnInstalled Then
'  wscript.echo "The service is in the " & chr(34) & CheckServiceStatus(strService) & chr(34) & " state."
  If NOT blnRunning Then
    blnReturn = MonitorService(strService)
'   Note: If you wish, you can use the blnReturn value to do some further testing.
  End If
End If

'Copy the files and templates
objFSO.CopyFile strScriptPath & "Bginfo.exe", strProgramFolder & "\", True
If isTerminalServer Then
  objFSO.CopyFile strScriptPath & "ResetWallpaper.vbs", strProgramFolder & "\", True
  objFSO.CopyFile strScriptPath & "TSBackdrop.exe", strProgramFolder & "\", True
  objFSO.CopyFile strScriptPath & strCitrixBGI & ".bgi", strProgramFolder & "\Standard.bgi", True
Else
  objFSO.CopyFile strScriptPath & "Bginfo.txt", strProgramFolder & "\", True
  objFSO.CopyFile strScriptPath & "Standard.bgi", strProgramFolder & "\", True
End If

If NOT isTerminalServer Then
' Run BGInfo, which will create the BGInfo.bmp file.
' This will then allow us to give users modify permissions on the file.
  strCommandLine = chr(34) & strProgramFolder & "\Bginfo.exe" & chr(34) & " " & chr(34) & strProgramFolder & "\standard.bgi" & chr(34) & " /timer:0 /NOLICPROMPT"
  WshShell.Run strCommandLine,1,True

' Check for the existence of the BGInfo.bmp file. If it doesn't exist, call the MonitorFileCreation
' subroutine and pass it the folder and file to monitor.
  If NOT objFSO.FileExists(strSystemRoot & "\BGInfo.bmp") Then
    Call MonitorFileCreation(strSystemRoot,"BGInfo.bmp")
  End If

' Set permissions on the BGInfo.bmp file to allow users to modify it.
' Note that you can either use xcacls.vbs or xcacls.exe here. I chose to use xcacls.vbs, which
' is automatically copied to each server during the automated server build. Also note that you
' use M for modify rights with xcacls.vbs, whilst xcacls.exe uses C.
  strCommandLine = "cscript.exe " & strSystemRoot & "\XCACLS.vbs " & strSystemRoot & "\BGInfo.bmp /E /G Users:M"
  WshShell.Run strCommandLine,1,True
End If

'Create the Start Menu Startup folder Shortcut
If isTerminalServer Then
  Set oShellLink = WshShell.CreateShortcut(strAUStartup & "\ResetWallpaper.lnk")
  oShellLink.TargetPath = chr(34) & strSystemRoot & "\system32\cscript.exe" & chr(34)
  oShellLink.Arguments = "//nologo " & chr(34) & strProgramFolder & "\ResetWallpaper.vbs" & chr(34)
  If objFSO.FileExists(strProgramFolder & "\Bginfo.exe") Then
    oShellLink.IconLocation = strProgramFolder & "\Bginfo.exe" & ",0"
  End If
Else
  Set oShellLink = WshShell.CreateShortcut(strAUStartup & "\BGInfo.lnk")
  oShellLink.TargetPath = chr(34) & strProgramFolder & "\Bginfo.exe" & chr(34)
  oShellLink.Arguments = chr(34) & strProgramFolder & "\standard.bgi" & chr(34) & " /timer:0 /NOLICPROMPT"
End If
oShellLink.WorkingDirectory = strProgramFolder
oShellLink.Save

Set WshShell = Nothing
Set objFSO = Nothing
Set objFolder = Nothing
Set oShellLink = Nothing

WScript.Quit(0)

Sub MonitorFileCreation(strDirectory, strFile)
Dim strFullPathandFile, strComputer, objWMIService, colMonitoredEvents
Dim objLatestEvent, strNewFile, arrNewFile, strFileName
  strFullPathandFile = strDirectory & "\" & strFile
  strDirectory = chr(34) & Replace(strDirectory,"\","\\\\") & chr(34)
  strComputer = "."
  Set objWMIService = GetObject("winmgmts:" _
    & "{impersonationLevel=impersonate}!\\" & _
        strComputer & "\root\cimv2")
  Set colMonitoredEvents = objWMIService.ExecNotificationQuery _
    ("SELECT * FROM __InstanceCreationEvent WITHIN 10 WHERE " _
        & "Targetinstance ISA 'CIM_DirectoryContainsFile' and " _
            & "TargetInstance.GroupComponent= " _
                & "'Win32_Directory.Name=" & strDirectory & "'")
  Do
    Set objLatestEvent = colMonitoredEvents.NextEvent
    strNewFile = objLatestEvent.TargetInstance.PartComponent
    arrNewFile = Split(strNewFile, "=")
    strFileName = arrNewFile(1)
    strFileName = Replace(strFileName, "\\", "\")
    strFileName = Replace(strFileName, Chr(34), "")
    If StrComp(strFileName,strFullPathandFile, 1) = 0 Then
      Exit Do
    End If
  Loop
  Set objWMIService = Nothing
  Set colMonitoredEvents = Nothing
  Set objLatestEvent = Nothing
End Sub

Function MonitorService(strService)
  Dim strComputer, SINK, objWMIService
  strComputer = "."
  Set SINK = WScript.CreateObject("WbemScripting.SWbemSink","SINK_")
  Set objWMIService = GetObject("winmgmts:" _
    & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
  objWMIService.ExecNotificationQueryAsync SINK, _
    "SELECT * FROM __InstanceModificationEvent WITHIN 1 " & _
    "WHERE TargetInstance ISA 'Win32_Service'"
  Do
    If lcase(CheckServiceStatus(strService)) = "running" Then
      MonitorService = True
      Exit Do
    End If
    MonitorService = False
    WScript.Sleep 1000
  Loop
  Set SINK = Nothing
  Set objWMIService = Nothing
End Function

Sub SINK_OnObjectReady(objLatestEvent, objAsyncContext)
' Trap asynchronous events.
  If LCase(objLatestEvent.TargetInstance.Name) = LCase(strService) Then
'    WScript.Echo "Status of the " & objLatestEvent.TargetInstance.Name & " service is: " & objLatestEvent.TargetInstance.State
  End If
End Sub

Function CheckServiceStatus(strService)
  Dim strComputer, objWMIService, colRunningServices, objService
  strComputer = "."
  Set objWMIService = GetObject("winmgmts:" _
    & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")
  Set colRunningServices = objWMIService.ExecQuery _
    ("Select * from Win32_Service where Name = '" & strService & "'")
  If colRunningServices.Count = 0 Then
    CheckServiceStatus = "Not Installed"
  End If
  For Each objService in colRunningServices
    CheckServiceStatus = objService.State
  Next
  Set objWMIService = Nothing
  Set colRunningServices = Nothing
End Function

Function isTerminalServer   
  Dim strComputer, objWMIService, colItems, objItem   
  strComputer = "."  
  Set objWMIService = GetObject("winmgmts:" _   
    & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2")  
  Set colItems = objWMIService.ExecQuery _  
    ("Select * from Win32_TerminalServiceSetting")  
  For Each objItem in colItems  
    Select Case objItem.LicensingType  
      Case "2"  
        isTerminalServer = True  
      Case "4"  
        isTerminalServer = True  
      Case Else  
        isTerminalServer = False  
    End Select
  Next  
  Set objWMIService = Nothing  
  Set colItems = Nothing  
End Function  
